#if !defined(EXPRESSION_GENERATOR_HH)
#define EXPRESSION_GENERATOR_HH

#include <math.h>

#include "AstManager.hh"

///////////////////////////////////////////////////////////////////////////
class ExpressionGenerator {
///////////////////////////////////////////////////////////////////////////

public :

  ExpressionGenerator( AstManager& ast, int depth, int maxDigits, int seed ) : 
    ast_(ast), depth_(depth) { 
    srand(seed);
    precision_ = (maxDigits > 0 ? (int)(pow(10,maxDigits)) : RAND_MAX);
  }
  ~ExpressionGenerator( void ) { }

  AstNode* operator()( void ) {
    return generateRecursively_(depth_);
  }

private :
  AstManager& ast_;
  int         depth_;
  int         precision_;

  AstNode* generateRecursively_(int depth) {
    AstNode* n;
    if (depth == 0) {
      n = ast_.newOperand();
      n->setValue(rand() % precision_);
    } else {
      n = ast_.newOperator();
      n->setOperand(AstNode::LEFT, generateRecursively_(depth - 1));
      n->setOpType(generateOpType_());
      n->setOperand(AstNode::RIGHT, generateRecursively_(depth - 1));
    }
    return n;
  }
  AstNode::OpType generateOpType_( void ) {
    int i = rand() % 4 + 1;
    switch (i) {
    case 1 : return AstNode::PLUS;
    case 2 : return AstNode::MINUS;
    case 3 : return AstNode::TIMES;
    case 4 : return AstNode::FRAC;
    }
  }

}; // End of ExpressionGenerator


#endif
