#if !defined(PARSER_HH)
#define PARSER_HH

#include <iostream>

#include "Lexer.hh"
#include "AstManager.hh"
#include "ErrorManager.hh"

///////////////////////////////////////////////////////////////////////////
class Parser {
///////////////////////////////////////////////////////////////////////////

public :

  Parser( Lexer& lex, AstManager& ast, ErrorManager& error ) : 
    lex_(lex), ast_(ast), error_(error) { }
  ~Parser( void ) { }
  
  AstNode* operator()( void ) {
    AstNode* expression = 0;
    expression = parseExpression_();
//     if (lex_.cur().getPartOfSpeech() != Token::EOFF) {
//       error_("Garbage after end of program");
//     }
    return expression;
  }  
  
private :
  Lexer&        lex_;
  AstManager&   ast_;
  ErrorManager& error_;
  
  AstNode* parseExpression_() {
    AstNode* e = 0;
    lex_.next();
    if (lex_.cur().getPartOfSpeech() == Token::NUMBER) {
      e = ast_.newOperand();
      e->setValue(atoi(lex_.cur().getLexeme().data()));
    } else if (lex_.cur().getPartOfSpeech() == Token::LPAR) {
      e = new Operator();
      e->setOperand(AstNode::LEFT, parseExpression_());
      if (e->getOperand(AstNode::LEFT) == 0) {
	error_("Missing expression");
      }
      e->setOpType(parseOpType_());
      if (e->getOpType() == AstNode::NOP) {
	error_("Missing operator");
      }
      e->setOperand(AstNode::RIGHT, parseExpression_());
      if (e->getOperand(AstNode::RIGHT) == 0) {
	error_("Missing expression");
      }
      lex_.next();      
      if (lex_.cur().getPartOfSpeech() != Token::RPAR) {
	error_("Missing )");
      }
    } 
    return e;
  }

  AstNode::OpType parseOpType_() {
    lex_.next();
    char ch = lex_.cur().getLexeme()[0];
    switch (ch) {
      case '+' : return AstNode::PLUS;
      case '-' : return AstNode::MINUS;
      case '*' : return AstNode::TIMES;
      case '/' : return AstNode::FRAC;
    default :
      return AstNode::NOP;
    }
  }

}; // End of class Parser

#endif
